# """ 
# REDCAP input CSV schema
# """ 

# from frictionless import Schema 

# schema = Schema({
#   "fields": [
#     {
#       "type": "string",
#       "name": "Variable / Field Name"
#     },
#     {
#       "type": "string",
#       "name": "Form Name"
#     },
#     {
#       "type": "string",
#       "name": "Section Header"
#     },
#     {
#       "type": "string",
#       "name": "Field Type"
#     },
#     {
#       "type": "string",
#       "name": "Field Label"
#     },
#     {
#       "type": "string",
#       "name": "Choices, Calculations, OR Slider Labels"
#     },
#     {
#       "type": "string",
#       "name": "Field Note"
#     },
#     {
#       "type": "string",
#       "name": "Text Validation Type OR Show Slider Number"
#     },
#     {
#       "type": "number",
#       "name": "Text Validation Min"
#     },
#     {
#       "type": "number",
#       "name": "Text Validation Max"
#     },
#     {
#       "type": "string",
#       "name": "Identifier?"
#     },
#     {
#       "type": "string",
#       "name": "Branching Logic (Show field only if...)"
#     },
#     {
#       "type": "string",
#       "name": "Required Field?"
#     },
#     {
#       "name": "Custom Alignment",
#       "type": "any"
#     },
#     {
#       "name": "Question Number (surveys only)",
#       "type": "any"
#     },
#     {
#       "type": "string",
#       "name": "Matrix Group Name"
#     }
#   ]
# })

